@@ -1,5 +1,6 @@
 {-# LANGUAGE EmptyDataDecls #-}
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE TypeSynonymInstances #-}
 {-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE TypeFamilies #-}
@@ -10,9 +11,11 @@
 {-# LANGUAGE KindSignatures #-}
 {-# LANGUAGE DatatypeContexts #-}
 {-# LANGUAGE DeriveFunctor #-}
+{-# LANGUAGE MagicHash #-}
 #if __GLASGOW_HASKELL__ >= 701
 {-# LANGUAGE DeriveGeneric #-}
 #endif
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Main (
   -- * Run all tests
@@ -22,32 +25,101 @@ module Main (
 import Prelude hiding (Either(..))
 import Generics.Deriving
 import Generics.Deriving.TH
-
+import GHC.Exts (Addr#, Char#, Double#, Float#, Int#, Word#)
+import qualified Text.Read.Lex (Lexeme)
 
 --------------------------------------------------------------------------------
 -- Temporary tests for TH generation
 --------------------------------------------------------------------------------
 
+data Empty a
+
 data (:/:) f a = MyType1Nil
                | MyType1Cons { myType1Rec :: (f :/: a), myType2Rec :: MyType2 }
-               | MyType1Cons2 (f :/: a) Int a (f a) 
+               | MyType1Cons2 (f :/: a) Int a (f a)
+               | (f :/: a) :/: MyType2
 
 #if __GLASGOW_HASKELL__ >= 701
-  deriving Generic
+  deriving ( Generic
+# if __GLASGOW_HASKELL__ >= 705
+           , Generic1
+# endif
+           )
 #endif
 
 data MyType2 = MyType2 Float ([] :/: Int)
+data PlainHash a = Hash a Addr# Char# Double# Float# Int# Word#
+
+#if __GLASGOW_HASKELL__ >= 701
+deriving instance Generic (Empty a)
+deriving instance Generic MyType2
+#endif
+
+#if __GLASGOW_HASKELL__ < 705
+$(deriveMeta ''Empty)
+$(deriveMeta ''(:/:))
+$(deriveMeta ''MyType2)
+#endif
 
 #if __GLASGOW_HASKELL__ < 701
+$(deriveRepresentable0 ''Empty)
+$(deriveRepresentable0 ''(:/:))
+$(deriveRepresentable0 ''MyType2)
+#endif
 
-$(deriveAll ''(:/:))
-$(deriveAll ''MyType2)
-
+#if __GLASGOW_HASKELL__ >= 705
+deriving instance Generic1 Empty
 #else
+$(deriveRepresentable1 ''Empty)
+$(deriveRepresentable1 ''(:/:))
+#endif
 
--- deriving instance Generic (f :/: a)
-deriving instance Generic MyType2
+#if __GLASGOW_HASKELL__ >= 711
+deriving instance Generic (PlainHash a)
+deriving instance Generic1 PlainHash
+#else
+$(deriveAll0And1 ''PlainHash)
+#endif
 
+-- Test to see if generated names are unique
+data Lexeme = Lexeme
+
+$(deriveAll ''Main.Lexeme)
+$(deriveAll ''Text.Read.Lex.Lexeme)
+
+#if __GLASGOW_HASKELL__ >= 703
+data family MyType3 a b
+newtype instance MyType3 ()   b = MyType3Newtype b
+data    instance MyType3 Bool b = MyType3True | MyType3False
+data    instance MyType3 Int  b = MyType3Hash b Addr# Char# Double# Float# Int# Word#
+
+# if __GLASGOW_HASKELL__ < 707
+$(deriveMeta 'MyType3Newtype)
+$(deriveMeta 'MyType3True)
+# endif
+
+# if __GLASGOW_HASKELL__ >= 705
+deriving instance Generic (MyType3 ()   b)
+deriving instance Generic (MyType3 Bool b)
+# else
+$(deriveRepresentable0 'MyType3Newtype)
+$(deriveRepresentable0 'MyType3True)
+# endif
+
+# if __GLASGOW_HASKELL__ >= 707
+deriving instance Generic1 (MyType3 ())
+deriving instance Generic1 (MyType3 Bool)
+# else
+$(deriveRepresentable1 'MyType3Newtype)
+$(deriveRepresentable1 'MyType3False)
+# endif
+
+# if __GLASGOW_HASKELL__ >= 711
+deriving instance Generic  (MyType3 Int b)
+deriving instance Generic1 (MyType3 Int)
+# else
+$(deriveAll0And1 'MyType3Hash)
+# endif
 #endif
 
 --------------------------------------------------------------------------------
@@ -101,7 +173,7 @@ instance Uniplate Tree where
   descend    = descenddefault
   descendM   = descendMdefault
   transform  = transformdefault
-  transformM = transformdefault
+  transformM = transformMdefault
 instance GEnum    Tree where genum      = genumDefault
 
 #endif
@@ -112,7 +184,7 @@ upgradeTree (Branch n l r) = Branch (succ n) l r
 
 -- Example usage
 tree = Branch 2 Empty (Branch 1 Empty Empty)
-testsTree = [ gshow tree 
+testsTree = [ gshow tree
             , gshow (children tree)
             , gshow (descend (descend (\_ -> Branch 0 Empty Empty)) tree)
             , gshow (context tree [Branch 1 Empty Empty,Empty])
@@ -123,7 +195,7 @@ testsTree = [ gshow tree
 -- Example: lists (kind * -> *)
 --------------------------------------------------------------------------------
 
-data List a = Nil | Cons a (List a) 
+data List a = Nil | Cons a (List a)
 
 #if __GLASGOW_HASKELL__ >= 701
 deriving instance Generic (List a)
@@ -178,7 +250,7 @@ instance (Uniplate a) => Uniplate (List a) where
   descend    = descenddefault
   descendM   = descendMdefault
   transform  = transformdefault
-  transformM = transformdefault
+  transformM = transformMdefault
 
 #else
 
@@ -222,14 +294,7 @@ $(deriveRepresentable0 ''Nested)
 #if __GLASGOW_HASKELL__ >= 705
 deriving instance Generic1 Nested
 #else
-
-type RepNested = D1 Nested_ (C1 Nested_Leaf_ U1 :+: C1 Nested_Nested_ (Par1 :*: Nested :.: Rec1 []))
-instance Generic1 Nested where
-  type Rep1 Nested = RepNested
-  from1 Leaf = M1 (L1 (M1 U1))
-  from1 (Nested a l) = M1 (R1 (M1 (Par1 a :*: Comp1 (gmap Rec1 l))))
-  to1 (M1 (L1 (M1 U1))) = Leaf
-  to1 (M1 (R1 (M1 (Par1 a :*: Comp1 l)))) = Nested a (gmap unRec1 l)
+$(deriveRepresentable1 ''Nested)
 #endif
 
 #if __GLASGOW_HASKELL__ < 701
@@ -240,7 +305,7 @@ instance (GShow a) => GShow (Nested a) where
 instance GFunctor Nested where
   gmap = gmapdefault
 
-#else 
+#else
 
 instance (GShow a) => GShow (Nested a)
 instance GFunctor Nested
@@ -296,7 +361,7 @@ instance Generic1 Rose where
 
 #endif
 
-#if __GLASGOW_HASKELL_ >= 701
+#if __GLASGOW_HASKELL__ >= 701
 
 instance (GShow a) => GShow (Rose a)
 instance GFunctor Rose
@@ -342,12 +407,11 @@ $(deriveRepresentable0 ''GRose)
 #if __GLASGOW_HASKELL__ >= 705
 deriving instance (Functor f) => Generic1 (GRose f)
 #else
-
-type Rep1GRose f = D1 GRose_ (C1 GRose_GRose_ (Rec1 f :*: f :.: (Rec1 (GRose f))))
-instance (GFunctor f) => Generic1 (GRose f) where
-  type Rep1 (GRose f) = Rep1GRose f
-  from1 (GRose a x) = M1 (M1 (Rec1 a :*: Comp1 (gmap Rec1 x)))
-  to1 (M1 (M1 (Rec1 a :*: Comp1 x))) = GRose a (gmap unRec1 x)
+$(deriveRep1 ''GRose)
+instance (Functor f) => Generic1 (GRose f) where
+  type Rep1 (GRose f) = $(makeRep1 ''GRose) f
+  from1 = $(makeFrom1 ''GRose)
+  to1 = $(makeTo1 ''GRose)
 #endif
 
 #if __GLASGOW_HASKELL__ < 701
@@ -355,7 +419,7 @@ instance (GFunctor f) => Generic1 (GRose f) where
 instance (GShow (f a), GShow (f (GRose f a))) => GShow (GRose f a) where
   gshowsPrec = gshowsPrecdefault
 
-instance (GFunctor f) => GFunctor (GRose f) where
+instance (Functor f, GFunctor f) => GFunctor (GRose f) where
   gmap = gmapdefault
 
 #else
@@ -445,7 +509,7 @@ instance Generic1 Weird where
 
 instance GFunctor Weird
 
-#else 
+#else
 
 instance GFunctor Weird where
   gmap = gmapdefault
@@ -473,15 +537,7 @@ $(deriveRepresentable0 ''Bush)
 #if __GLASGOW_HASKELL__ >= 705
 deriving instance Generic1 Bush
 #else
-
-type Rep1Bush = U1 :+: Par1 :*: Bush :.: Rec1 Bush
-instance Generic1 Bush where
-  type Rep1 Bush = Rep1Bush
-  from1 BushNil = L1 U1
-  from1 (BushCons a b) = R1 (Par1 a :*: Comp1 (gmap Rec1 b))
-  to1 (L1 U1) = BushNil
-  to1 (R1 (Par1 a :*: Comp1 b)) = BushCons a (gmap unRec1 b)
-
+$(deriveRepresentable1 ''Bush)
 #endif
 
 #if __GLASGOW_HASKELL__ < 701
